///////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// Dummy author
// Dummy company
//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third
// parties from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 1998-1999.
//
// This notice must be included in all copies or derivative works.
//
// Applicable File Name:  DummyTypeCS.cpp
//

#include "Descriptors.h"
#include "CodingSchemes.h"
#include "Encoders.h"
#include <stdio.h>
#include <sys/types.h>
#ifndef WIN32
#include <netinet/in.h>
#endif
 

using namespace XM;

const UUID DummyTypeCSInterface::myID = UUID("");
const char *DummyTypeCSInterface::myName =
                         "DummyTypeCodingSchemeInterface";

// create a UUID for your type
const UUID DummyTypeCS::myID = UUID("to be created with xx");
const char *DummyTypeCS::myName = "DummyTypeCodingScheme";

const UUID DummyTypeCS::valID = UUID("");
const char *DummyTypeCS::valName = "";

const UUID DummyTypeCSInterfaceABC::myID = UUID("");

DummyTypeCSInterfaceABC::DummyTypeCSInterfaceABC()
{ SetInterfaceID(myID);}

//=============================================================================
DummyTypeCSInterface::DummyTypeCSInterface(DummyTypeCS *aDummyType):
m_DummyTypeCS(aDummyType)
{
	SetInterfaceID(myID);
}

//----------------------------------------------------------------------------
DummyTypeCSInterface::~DummyTypeCSInterface()
{
}


//----------------------------------------------------------------------------
const UUID& DummyTypeCSInterface::GetInterfaceID(void)
{ return myID; }

//----------------------------------------------------------------------------
const char *DummyTypeCSInterface::GetName(void)
{ return myName; }

//----------------------------------------------------------------------------
void DummyTypeCSInterface::destroy(void)
{ delete m_DummyTypeCS; }

//----------------------------------------------------------------------------
EncoderFileIO *DummyTypeCSInterface::GetEncoderStreamBuffer(void)
{
  return m_DummyTypeCS->GetEncoderStreamBuffer();
}

//----------------------------------------------------------------------------
int DummyTypeCSInterface::SetEncoderStreamBuffer(EncoderFileIO *aBuffer)
{
  return m_DummyTypeCS->SetEncoderStreamBuffer(aBuffer);
}

//----------------------------------------------------------------------------
DecoderFileIO *DummyTypeCSInterface::GetDecoderStreamBuffer(void)
{
  return m_DummyTypeCS->GetDecoderStreamBuffer();
}

//----------------------------------------------------------------------------
int DummyTypeCSInterface::SetDecoderStreamBuffer(DecoderFileIO *aBuffer)
{
  return m_DummyTypeCS->SetDecoderStreamBuffer(aBuffer);
}

//----------------------------------------------------------------------------
DummyTypeDescriptorInterfaceABC
*DummyTypeCSInterface::GetDescriptorInterface(void)

{
  return m_DummyTypeCS->GetDescriptorInterface();
}

//----------------------------------------------------------------------------
int DummyTypeCSInterface::
SetDescriptorInterface(DummyTypeDescriptorInterfaceABC
		       *aDummyTypeDescriptorInterface)
{
  return m_DummyTypeCS->
    SetDescriptorInterface(aDummyTypeDescriptorInterface);
}

//----------------------------------------------------------------------------
int DummyTypeCSInterface::StartEncode(void)
{
  return m_DummyTypeCS->StartEncode();
}

//----------------------------------------------------------------------------
int DummyTypeCSInterface::StartDecode(void)
{
  return m_DummyTypeCS->StartDecode();
}
//=============================================================================
DummyTypeCS::DummyTypeCS():
m_Interface(this),
m_isProprietary(false),
m_EncoderBuffer(0),
m_DecoderBuffer(0),
m_DescriptorInterface(0)
{

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* create sub CS and
     connect them with descriptor if availabe*/
  SubDummyTypeACS *aSubDummyTypeACS= new SubDummyTypeACS();
  SubDummyTypeBCS *aSubDummyTypeBCS= new SubDummyTypeBCS();

  /* connect sub CS with DummyTypeCS*/
  if (m_SubDummyTypeACS)
    m_SubDummyTypeACS=aSubDummyTypeACS->GetInterface();
  //    SetSubDummyTypeACSInterface(aSubDummyTypeACS->GetInterface());
  if (m_SubDummyTypeBCS)
    m_SubDummyTypeBCS=aSubDummyTypeBCS->GetInterface();
  //  SetSubDummyTypeBCSInterface(aSubDummyTypeBCS->GetInterface());
#endif /* __HasSubTypes*/

  SetExposedInterface(&m_Interface);
}
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
#endif /* __HasSubTypes*/

//----------------------------------------------------------------------------
DummyTypeCS::DummyTypeCS(DummyTypeDescriptorInterfaceABC* aDummyType):
m_Interface(this),
m_isProprietary(false),
m_EncoderBuffer(NULL),
m_DecoderBuffer(NULL),
m_DescriptorInterface(NULL)
{
  /* create descriptor if it does not exist*/
  if (!aDummyType) {
    DummyTypeDescriptor *descriptor =
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
      new DummyTypeDescriptor(0,0); /* one argument for each sub-component*/
#else
      new DummyTypeDescriptor;
#endif
      aDummyType=descriptor->GetInterface();
  }

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
#ifdef VERBOSE
  fprintf(stderr,"Connect sub descriptors\n");
#endif

  /* create sub CS and
     connect them with descriptor if availabe*/
  {
    SubDummyTypeACS *aSubDummyTypeACS;

    if (aDummyType->GetSubDummyTypeADescriptorInterface()) {
      aSubDummyTypeACS= new
	SubDummyTypeACS(aDummyType->GetSubDummyTypeADescriptorInterface());
    }
    else { 
      aSubDummyTypeACS= new SubDummyTypeACS();
    }
    if (aSubDummyTypeACS) {
      m_SubDummyTypeACS=aSubDummyTypeACS->GetInterface();
      //      SetSubDummyTypeACSInterface(m_SubDummyTypeACS);
    }
  }

  {
    SubDummyTypeBCS *aSubDummyTypeBCS;

    if (aDummyType->GetSubDummyTypeBDescriptorInterface()) {
      aSubDummyTypeBCS= new
	SubDummyTypeBCS(aDummyType->GetSubDummyTypeBDescriptorInterface());
    }
    else { 
      aSubDummyTypeBCS= new SubDummyTypeBCS();
    }
    if (aSubDummyTypeBCS) {
      m_SubDummyTypeBCS=aSubDummyTypeBCS->GetInterface();
      //      SetSubDummyTypeBCSInterface(m_SubDummyTypeBCS);
    }
  }
#endif /* __HasSubTypes*/

  /* connect descritors with CS*/
  SetDescriptorInterface(aDummyType);

  SetExposedInterface(&m_Interface);
}

//----------------------------------------------------------------------------
DummyTypeCS::~DummyTypeCS()
{
  /* release descriptor*/
  if (m_DescriptorInterface)
    m_DescriptorInterface->release();
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  if (m_SubDummyTypeACS) m_SubDummyTypeACS->destroy();
  if (m_SubDummyTypeBCS) m_SubDummyTypeBCS->destroy();
#endif /* __HasSubTypes*/
}

//----------------------------------------------------------------------------
const UUID& DummyTypeCS::GetObjectID(void)
{ return myID; }

//----------------------------------------------------------------------------
const char *DummyTypeCS::GetName(void)
{ return myName; }

//----------------------------------------------------------------------------
const UUID& DummyTypeCS::GetValueID(void)
{ return valID; }

//----------------------------------------------------------------------------
const char* DummyTypeCS::GetValueName(void)
{ return valName; }

//----------------------------------------------------------------------------
bool DummyTypeCS::IsProprietary(void)
{ return m_isProprietary; }


//----------------------------------------------------------------------------
EncoderFileIO * DummyTypeCS::GetEncoderStreamBuffer(void)
{
  return m_EncoderBuffer;
}

//----------------------------------------------------------------------------
int DummyTypeCS::SetEncoderStreamBuffer(EncoderFileIO *aBuffer)
{
  if (!aBuffer) return -1;

  m_EncoderBuffer = aBuffer;
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  if (m_SubDummyTypeACS) m_SubDummyTypeACS->SetEncoderStreamBuffer(aBuffer);
  if (m_SubDummyTypeBCS) m_SubDummyTypeBCS->SetEncoderStreamBuffer(aBuffer);
#endif /* __HasSubTypes*/
  return 0;
}

//----------------------------------------------------------------------------
DecoderFileIO * DummyTypeCS::GetDecoderStreamBuffer(void)
{
  return m_DecoderBuffer;
}

//----------------------------------------------------------------------------
int DummyTypeCS::SetDecoderStreamBuffer(DecoderFileIO *aBuffer)
{
  if (!aBuffer) return -1;

  m_DecoderBuffer = aBuffer;
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  if (m_SubDummyTypeACS) m_SubDummyTypeACS->SetDecoderStreamBuffer(aBuffer);
  if (m_SubDummyTypeBCS) m_SubDummyTypeBCS->SetDecoderStreamBuffer(aBuffer);
#endif /* __HasSubTypes*/

  return 0;
}

//----------------------------------------------------------------------------
DummyTypeDescriptorInterfaceABC *DummyTypeCS::
GetDescriptorInterface(void)
{
  return m_DescriptorInterface;
}

//----------------------------------------------------------------------------
int DummyTypeCS::
SetDescriptorInterface(DummyTypeDescriptorInterfaceABC
		       *aDummyTypeDescriptorInterface)
{
  /* check if new value is different from old value*/
  if (m_DescriptorInterface == 
      aDummyTypeDescriptorInterface) return 0;

  /* release old interface*/
  if (m_DescriptorInterface) 
    m_DescriptorInterface->release();

  /* add new interface*/
  m_DescriptorInterface = 
    aDummyTypeDescriptorInterface;
  if (m_DescriptorInterface) {
    m_DescriptorInterface->addref();

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
    /* conect sub descriptors with sub CS*/
    if ((m_DescriptorInterface->
	 GetSubDummyTypeADescriptorInterface()) &&
	m_SubDummyTypeACS) {
      m_SubDummyTypeACS->
	SetDescriptorInterface(m_DescriptorInterface->
			       GetSubDummyTypeADescriptorInterface());
    }
    if ((m_DescriptorInterface->
	 GetSubDummyTypeBDescriptorInterface()) &&
	m_SubDummyTypeBCS) {
      m_SubDummyTypeBCS->
	SetDescriptorInterface(m_DescriptorInterface->
			       GetSubDummyTypeBDescriptorInterface());
    }
#endif /* __HasSubTypes*/
  }
  else {
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
    /* if no descriptor, release also descriptor interfaces from
       sub CS*/
    m_SubDummyTypeACS->SetDescriptorInterface(0);
    m_SubDummyTypeBCS->SetDescriptorInterface(0);
#endif /* __HasSubTypes*/
    return -1;
  }

  return 0;
}

//----------------------------------------------------------------------------
// This writes the contents of the D/DSType to file as a sequence of bits
int DummyTypeCS::StartEncode(void)
{
  long v;

  // Is the file IO set up? If not return
  if(m_EncoderBuffer==NULL) {
    fprintf(stderr,"DummyTypeCS::StartEncode: Buffer not initialised\n");
    return(-1);
  }

  // Is the interface set up?
  if(m_DescriptorInterface==NULL) {
    fprintf(stderr,"DummyTypeCS::StartEncode: Interface not initialised\n");
    return -1;
  }

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* check values for SubDummyTypeA **************************/ 
  /* check if SubDummyTypeA descriptor is linked to DummyTypeA*/
  if (!m_DescriptorInterface->
      GetSubDummyTypeADescriptorInterface()) {
    return -1;
  }

  /* check if SubDummyTypeA descritor and stream buffer are linked
     to SubDummyTypeA CS*/
  if (!m_SubDummyTypeACS->GetDescriptorInterface() ||
      !m_SubDummyTypeACS->GetEncoderStreamBuffer()) return -1;

  /* check if SubDummyTypeA descriptor attached to
     DummyType descriptor
     is equal to SubDummyTypeA descritor attached to
     SubDummyTypeA CS */
  if (m_SubDummyTypeACS->GetDescriptorInterface() !=
      m_DescriptorInterface->
      GetSubDummyTypeADescriptorInterface()) return -1;

  /* check values for SubDummyTypeB **************************/ 
  /* check if SubDummyTypeB descriptor is linked to DummyTypeB*/
  if (!m_DescriptorInterface->
      GetSubDummyTypeBDescriptorInterface()) {
    return -1;
  }

  /* check if SubDummyTypeB descritor and stream buffer are linked
     to SubDummyTypeB CS*/
  if (!m_SubDummyTypeBCS->GetDescriptorInterface() ||
      !m_SubDummyTypeBCS->GetEncoderStreamBuffer()) return -1;

  /* check if SubDummyTypeB descriptor attached to
     DummyType descriptor
     is equal to SubDummyTypeB descritor attached to
     SubDummyTypeB CS */
  if (m_SubDummyTypeBCS->GetDescriptorInterface() !=
      m_DescriptorInterface->
      GetSubDummyTypeBDescriptorInterface()) return -1;

#endif /* __HasSubTypes*/

  // In this simple case I'll just get the value from the D/DSType
  v=GetDescriptorInterface()->GetDummyContents();

  // And now I'll dump it into the buffer. This says put the 32 bits
  // of the number stored in v to file.
  m_EncoderBuffer->PutBitsLong(v,32);

  /* style for puting the bits interpreted as some kind of data*/
  /* m_EncoderBuffer->PutBits(reinterpret_cast<char*> (&v),8*sizeof(long));*/

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* encode SubDummyTypeA description*/
  m_SubDummyTypeACS->StartEncode();

  /* encode SubDummyTypeB description*/
  m_SubDummyTypeBCS->StartEncode();
#endif /* __HasSubTypes*/

  // All done
  return 0;
}

//----------------------------------------------------------------------------
// This reads the contents of the D/DSType from file
int DummyTypeCS::StartDecode(void)
{
  long v;

  // Is the file IO set up? If not return
  if(m_DecoderBuffer==NULL) {
    fprintf(stderr,"DummyTypeCS::StartDecode: Buffer not initialised\n");
    return(-1);
  }

  // Is the interface set up?
  if(m_DescriptorInterface==NULL) {
    fprintf(stderr,"DummyTypeCS::StartDecode: Interface not initialised\n");
    return -1;
  }

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* check values for SubDummyTypeA **************************/ 
  /* check if SubDummyTypeA descriptor is linked to DummyTypeA*/
  if (!m_DescriptorInterface->
      GetSubDummyTypeADescriptorInterface()) {
    return -1;
  }

  /* check if SubDummyTypeA descritor and stream buffer are linked
     to SubDummyTypeA CS*/
  if (!m_SubDummyTypeACS->GetDescriptorInterface() ||
      !m_SubDummyTypeACS->GetDecoderStreamBuffer()) return -1;

  /* check if SubDummyTypeA descriptor attached to
     DummyType descriptor
     is equal to SubDummyTypeA descritor attached to
     SubDummyTypeA CS */
  if (m_SubDummyTypeACS->GetDescriptorInterface() !=
      m_DescriptorInterface->
      GetSubDummyTypeADescriptorInterface()) return -1;

  /* check values for SubDummyTypeB **************************/ 
  /* check if SubDummyTypeB descriptor is linked to DummyTypeB*/
  if (!m_DescriptorInterface->
      GetSubDummyTypeBDescriptorInterface()) {
    return -1;
  }

  /* check if SubDummyTypeB descritor and stream buffer are linked
     to SubDummyTypeB CS*/
  if (!m_SubDummyTypeBCS->GetDescriptorInterface() ||
      !m_SubDummyTypeBCS->GetDecoderStreamBuffer()) return -1;

  /* check if SubDummyTypeB descriptor attached to
     DummyType descriptor
     is equal to SubDummyTypeB descritor attached to
     SubDummyTypeB CS */
  if (m_SubDummyTypeBCS->GetDescriptorInterface() !=
      m_DescriptorInterface->
      GetSubDummyTypeBDescriptorInterface()) return -1;

#endif /* __HasSubTypes*/

  // Read 32 bits the data in from the bit stream
  /* style for reading the bits interpreted as nummeric data type*/
   m_DecoderBuffer->GetBitsLong(&v,8*sizeof(long));

  /* style for reading the bits interpreted as some kind of data type*/
  /* m_DecoderBuffer->GetBits(reinterpret_cast<char*> (&v),8*sizeof(long));*/

  // Tell it to move along to the next bit of data - isn't a part of GetBits!
  // to allow variable length decoding
  m_DecoderBuffer->TakeBits(8*sizeof(long));

  // Put the data into the D/DSType
  GetDescriptorInterface()->SetDummyContents(v);

#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  /* encode SubDummyTypeA description*/
  m_SubDummyTypeACS->StartDecode();

  /* encode SubDummyTypeB description*/
  m_SubDummyTypeBCS->StartDecode();

#endif /* __HasSubTypes*/

  // All done
  return 0;
}

//----------------------------------------------------------------------------
DummyTypeCSInterfaceABC *DummyTypeCS::GetInterface(void)
{ return &m_Interface; }



