///////////////////////////////////////////////////////////////////////////////
//
// This software module was originally developed by
//
// Dummy author
// Dummy company
//
// in the course of development of the MPEG-7 Experimentation Model.
//
// This software module is an implementation of a part of one or more MPEG-7
// Experimentation Model tools as specified by the MPEG-7 Requirements.
//
// ISO/IEC gives users of MPEG-7 free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to MPEG-7.
//
// Those intending to use this software module in hardware or software products
// are advised that its use may infringe existing patents. The original
// developer of this software module and his/her company, the subsequent
// editors and their companies, and ISO/IEC have no liability for use of this
// software module or modifications thereof in an implementation.
//
// Copyright is not released for non MPEG-7 conforming products. The
// organizations named above retain full right to use the code for their own
// purpose, assign or donate the code to a third party and inhibit third
// parties from using the code for non MPEG-7 conforming products.
//
// Copyright (c) 1998-1999.
//
// This notice must be included in all copies or derivative works.
//
// Applicable File Name:  DummyTypeSearch.h
//

#ifndef __DUMMYTYPESEARCH_H__
#define __DUMMYTYPESEARCH_H__

#include "XMInterfaces.h"                            /* Naming Convention */
#include "XMObjects.h"                               /* Naming Convention */
#include "Descriptors.h"
#include "DescriptionSchemes.h"

//=============================================================================

namespace XM
{


// Forward Declarations:
class DummyTypeSearchTool;

//=============================================================================
class DummyTypeSearchInterfaceABC: public I_SearchInterfaceABC
{
public:
  DummyTypeSearchInterfaceABC();
  virtual ~DummyTypeSearchInterfaceABC() {};

  virtual void destroy()=0;

  virtual int SetRefDescriptorInterface
    (DummyTypeDescriptorInterfaceABC
     *aDummyTypeDescriptorInterface) = 0;
  virtual DummyTypeDescriptorInterfaceABC*
    GetQueryDescriptorInterface(void) = 0;
  virtual int SetQueryDescriptorInterface
    (DummyTypeDescriptorInterfaceABC
     *aDummyTypeDescriptorInterface) = 0;

  virtual double GetDistance(void) = 0;
	
  /* sub-components do not have get and set functions because
     there is only one output value*/

  static const UUID myID;
};

//=============================================================================
class DummyTypeSearchInterface: 
public DummyTypeSearchInterfaceABC
{
public:
  DummyTypeSearchInterface(DummyTypeSearchTool* aTool);
  virtual ~DummyTypeSearchInterface();
	
  virtual const UUID& GetInterfaceID(void);
  virtual const char *GetName(void);

  virtual void destroy();

  virtual int SetRefDescriptorInterface
    (DummyTypeDescriptorInterfaceABC
     *aDummyTypeDescriptorInterface);
  virtual DummyTypeDescriptorInterfaceABC*
    GetQueryDescriptorInterface(void);
  virtual int SetQueryDescriptorInterface
    (DummyTypeDescriptorInterfaceABC
     *aDummyTypeDescriptorInterface);

  virtual double GetDistance(void);


  static const UUID myID;
  static const char *myName;

private:
  DummyTypeSearchTool *m_SearchTool;
};

//=============================================================================
class DummyTypeSearchTool: public Search
{
friend DummyTypeSearchInterface;
public:
  DummyTypeSearchTool();
  DummyTypeSearchTool(DummyTypeDescriptorInterfaceABC
		      *aQueryDescriptorInterface);

  virtual const UUID& GetObjectID(void);
  virtual const char *GetName(void);

  virtual DummyTypeSearchInterfaceABC *GetInterface(void);

  virtual bool SupportsPush(void);
  virtual bool SupportsPull(void);

  virtual int SetRefDescriptorInterface
    (DummyTypeDescriptorInterfaceABC
     *aDummyTypeDescriptorInterface);
  virtual DummyTypeDescriptorInterfaceABC*
    GetQueryDescriptorInterface(void);
  virtual int SetQueryDescriptorInterface
    (DummyTypeDescriptorInterfaceABC
     *aDummyTypeDescriptorInterface);

  virtual double GetDistance(void);

  static const UUID myID;
  static const char * myName;
private:
  virtual ~DummyTypeSearchTool();

  DummyTypeSearchInterface m_Interface;
  DummyTypeDescriptorInterfaceABC *m_RefDescriptorInterface;
  DummyTypeDescriptorInterfaceABC *m_QueryDescriptorInterface;
#ifdef __HasSubTypes /*include this section if sub descriptors exist,
		       remove this section if no sub-descriptors exist*/
  SubDummyTypeASearchInterfaceABC *m_SubDummyTypeASearch;
  SubDummyTypeBSearchInterfaceABC *m_SubDummyTypeBSearch;
#endif /* __HasSubTypes*/
};

};

#endif //__DUMMYTYPESEARCH_H__
